//+-------------------------------------------------------------------------------------------+
//|                                                                                           |
//|                                  SonicR Trade Levels.mq4                                  | 
//|                                                                                           |
//+-------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2013 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview, _v2.30g:

This indicator shows labeled lines on the chart for those EP (Entry), TP (Take Profit) and SL
(Stop Loss) trade prices that you enter into External Inputs.  You can enter thirty EPs,
one TP and one SL.  When there is more than one EP, an average EP labeled line is added, and 
it and the TP and the SL are automatically shifted to the right to help preclude overlapping.

Chart_Zoom Number-
   Placement of the end of lines and the start of labels requires different hard code values 
   depending on which of the six MT4 chart zoom views the chart is actually set at for display.  
   We shall refer to the six MT4 chart zoom views as requiring a Chart_Zoom number setting of 
   1-6, with the MT4 default Chart_Zoom being #3.  The Chart_Zoom number is needed by the hard 
   code to properly place the end of lines and the start of labels for display.  If this number 
   is out of range, code will deinitialize the indicator and you will have to open the External 
   Inputs window and enter a proper Chart_Zoom number. There are three options for getting the 
   required Chart_Zoom number:
   
   Chart_Zoom_Number_01234567 = "0"
   The number is imported from an indicator higher on the chart list of indicators, if there
   is one that exports it per the same Global Variable parameters that this indicator uses.
   Such an indicator could be the SonicR Filled Dragon indicator, or the SonicR PVA Candles
   indicator.
   
   Chart_Zoom_Number_01234567 = "1-6"
   The number is set and locked by this manual input.  All size charts on all size monitors 
   will properly display the bar widths.  When you wish to zoom in/out on your chart do this:
    
      A. First, enter the "Enter_Chart_Zoom_0123456" input (1-6) you want, based on the 
         chart zoom view you intend to change to.
      B. Second, use the MT4 icons "+/-" to zoom the chart in/out, per the input you made.
      C. Third, switch chart TF at least once to activate the change to the hard coded 
         variables used to produce the proper display.
         
   Chart_Zoom_Number_01234567 = "7"  
   The number is "AutoSensed", meaning  the hard code determines the Chart_Zoom number by
   comparing how many bars per chart there are to some hard coded parameters.  A disadvantage
   of this method is that if you use downsized multiple charts in the MT4 main window, or if
   you use an extra wide monitor, you create a problem for this bars per chart comparison, and 
   the results will be incorrect.  When you wish to zoom in/out on your chart, maximize the 
   chart to normal size first, and then do this:
   
      A. First, use the MT4 icons "+/-" to zoom the chart in/out to the chart zoom view that
         you wish to display.
      B. Second, switch the chart TF at least once to activate the change to the hard coded 
         variables used to produce the proper display.  You will also have to perform this
         step on each MT4 chart when you first open the MT4 application. If you use downsized
         charts, after performing A and B, you can downsize them again.

Displaying Manually Entered SL, TP and EPs-
   You must enter EPs sequentially.  Do not skip a number, leaving it with a zero value 
   between other inputs with non-zero values.  This will result in incorrect calculation of 
   the average and a display of the average showing an incorrect number of averaged entries.
   You must also designate if the trade is long or short.  You can select to show the manually
   entered SL and TP in the Autofill mode if you have not yet set those values in your MT4
   orders.

Displaying AutoFilled SL, TP and EPs-
   You can elect to bypass manually filling in the SL, TP and EPs values.  Hard code will get
   these values from the MT4 orders placed.  If there are differences between manually entered
   values and actual MT4 orders placed, the manual and the AutoFill displays will be different.
   If your MT4 orders do not include a TP and/or a SL, but you have manually entered them, then
   the manually entered values can be displayed in the AutoFill mode.  This enables you to use
   the AutoFill mode, displaying your "mental" TP and SL, but keeping them  off the broker 
   server until you desire to actually place them as pending orders.  You can set a time limit 
   that starts when the trade is closed, that the trade datum will continue to be displayed on 
   the chart, to allow you to take a picture of the trade even after it has been closed.  If a 
   new trade is entered on the same pair during this time, the original trade datum will be 
   lost.  This time span is entered in minutes and is set to a default of 60 minutes.  When a
   trade is closed, the TP/SL level will show the profit/loss, even if you manually closed the
   trade without ever setting a TP/SL.

Displaying the Potential Profit/Loss-
   Potential profit is displayed after the TP_1 label.  Potential loss is displayed after the
   SL_1 label.  These displayed values are rounded to the nearest whole pip.  If there are
   differences between the manually entered TP_1 and SL_1 values and the TP/SL values in MT4 
   orders placed, then the manual and the AutoFill displays of the potential profit/loss will 
   be different.  If Autofill is "false" then the manually entered TP_1 and SL_1 values are 
   automatically displayed and you must also manually enter if the trade is long or short.
   When the trade is closed, this label no longer displays.
   
Displaying the Current Profit/Loss-
   The current profit or loss is displayed at the left edge of the chart at the bid line and
   changes as the bid price changes.  This display is rounded off to the nearest fractional pip.  
  
Arrows-
   Arrows are displayed on each EP line where the entry was made, and also where the trade is
   closed.  These arrows are featured only in the AutoFill mode.

Chart Properties Settings-
   To see things displayed as intended, be sure in chart Properties on the Common tab you 
   have unchecked "Chart on foreground" and have checked "Candlesticks".

Changes from previous release 05-25-2013 to current release xx-xx-2013:  
01 - Increased the number of allowed EPs to 30, reduced TPs and SLs to just one each.
02 - Coded line terminations and label positions to be based on the Chart_Zoom setting that
     can be imported from the SonicR Filled Dragon indicator.
03 - Added "long" and "short" labels for single EP and average EP.
03 - Added Profit/Loss numbers to TP/SL labels.
04 - Added Current P/L display on Bid Line with Background box to enhance legibility.
05 - Added arrow marking trade close.
06 - Removed price alert.
                                                                   
                                                                     - Traderathome, 07-29-2013
-----------------------------------------------------------------------------------------------
Acknowledgements:

My compliments to St3ve and emmanuel7788 who have, thru their contributions, helped transform
this from a simple indicator originated to keep instructional charts clean and clear, into a
unique and powerful trade management tool for all Sonic R. System traders.

St3ve -        contributions to capturing the EP, SL and TP datum from MT4 orders, for coding
               the EPs to close price connecting lines when a trade is closed, and for coding
               that enables the user to continue to display the trade datum after the trade is
               closed.           
emmanuel7788 - contribution by way of providing sample coding for displaying EP arrows where 
               MT4 entry orders were made.
                                                              
-----------------------------------------------------------------------------------------------
Suggested Colors:            White Chart        Black Chart      

EP_Color                     MediumBlue         C'091,142,236' 
EP_Av_Color                  DarkOrchid         MediumOrchid;
SL_Color                     C'213,000,000'     C'226,035,035'  
TP_Color                     C'000,113,065'     C'016,186,092'
PL_Background_Color          C'255,255,155'     C'255,255,155'
Arrow long                   Blue               DodgerBlue
Arrow short                  OrangeRed          OrangeRed
Arrow closed                 Black              White
---------------------------------------------------------------------------------------------*/

//+-------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                   |                                                        
//+-------------------------------------------------------------------------------------------+ 
#property indicator_chart_window 

//Global External Inputs
extern bool   Indicator_On                    = true;
extern int    Chart_Zoom_Number_01234567      = 0;
extern bool   AutoFill_vs_Manual_Inputs       = true;
extern string AutoFill_Datum_Input            = "=============================";
extern bool   __Apply_Manual_SL_TP            = false;
extern bool   __Show_EP_to_Close_Lines        = true;
extern int    ____Show_Lines_for_Minutes      = 60;
extern bool   __Show_Entry_and_Exit_Arrows    = true;
extern color  __Arrow_Trade_Long_Color        = Blue;
extern color  __Arrow_Trade_Short_Color       = Crimson;
extern color  __Arrow_Trade_Closed_Color      = Black;
extern string Manual_Datum_Input              = "=============================";
extern bool   __Trade_is_Long_vs_Short        = true;
extern double SL_1                            = 0;
extern double TP_1                            = 0;
extern double EP_1                            = 0;
extern double EP_2                            = 0;
extern double EP_3                            = 0;
extern double EP_4                            = 0;
extern double EP_5                            = 0;
extern double EP_6                            = 0;
extern double EP_7                            = 0;
extern double EP_8                            = 0;
extern double EP_9                            = 0;
extern double EP_10                           = 0;
extern double EP_11                           = 0;
extern double EP_12                           = 0;
extern double EP_13                           = 0;
extern double EP_14                           = 0;
extern double EP_15                           = 0;
extern double EP_16                           = 0;
extern double EP_17                           = 0;
extern double EP_18                           = 0;
extern double EP_19                           = 0;
extern double EP_20                           = 0;
extern double EP_21                           = 0;
extern double EP_22                           = 0;
extern double EP_23                           = 0;
extern double EP_24                           = 0;
extern double EP_25                           = 0;
extern double EP_26                           = 0;
extern double EP_27                           = 0;
extern double EP_28                           = 0;
extern double EP_29                           = 0;
extern double EP_30                           = 0;
extern string Label_and_Line_Colors           = "=============================";
extern color  EP_Color                        = MediumBlue;  
extern color  EP_Av_Color                     = DarkOrchid;
extern color  SL_Color                        = C'213,000,000';  
extern color  TP_Color                        = C'000,113,065';
extern color  Current_PL_Background_Color     = C'255,255,185';
extern string Chart_TF_Range_of_Display       = "=============================";
extern int    Display_Min_TF                  = 1;
extern int    Display_Max_TF                  = 43200;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

//Buffers, Constants and Variables
bool          Deinitialized;
bool          ClosedFlag; 
double        Avg_Price, Factor, PositionProfit, PositionLoss, TP, SL,
              Poin, CurrentPL, orderList[30][2];
int           orders,i,k,s,obj_total,S1,S2,T0,T1,T2,T3,T4,T5,T6,T7,Trades,historyTotal;
string        EPstr,name,spc3,spc4;
string        spc  = "                                 "; //34 
string        spc2 = "                         "; //26
string        name01 = "[Trade] EP1 Line";
string        name02 = "[Trade] EP1 Label";
string        name03 = "[Trade] EP2 Line";
string        name04 = "[Trade] EP2 Label";
string        name05 = "[Trade] EP3 Line";
string        name06 = "[Trade] EP3 Label";
string        name07 = "[Trade] EP4 Line";
string        name08 = "[Trade] EP4 Label";
string        name09 = "[Trade] EP5 Line";
string        name10 = "[Trade] EP5 Label";
string        name11 = "[Trade] EP6 Line";
string        name12 = "[Trade] EP6 Label";
string        name13 = "[Trade] EP7 Line";
string        name14 = "[Trade] EP7 Label";
string        name15 = "[Trade] EP8 Line";
string        name16 = "[Trade] EP8 Label";
string        name17 = "[Trade] EP9 Line";
string        name18 = "[Trade] EP9 Label";
string        name19 = "[Trade] EP10 Line";
string        name20 = "[Trade] EP10 Label";
string        name21 = "[Trade] EP11 Line";
string        name22 = "[Trade] EP11 Label";
string        name23 = "[Trade] EP12 Line";
string        name24 = "[Trade] EP12 Label";
string        name25 = "[Trade] EP13 Line";
string        name26 = "[Trade] EP13 Label";
string        name27 = "[Trade] EP14 Line";
string        name28 = "[Trade] EP14 Label";
string        name29 = "[Trade] EP15 Line";
string        name30 = "[Trade] EP15 Label";
string        name31 = "[Trade] EP16 Line";
string        name32 = "[Trade] EP16 Label";
string        name33 = "[Trade] EP17 Line";
string        name34 = "[Trade] EP17 Label";
string        name35 = "[Trade] EP18 Line";
string        name36 = "[Trade] EP18 Label";
string        name37 = "[Trade] EP19 Line";
string        name38 = "[Trade] EP19 Label";
string        name39 = "[Trade] EP20 Line";
string        name40 = "[Trade] EP20 Label";
string        name41 = "[Trade] EP21 Line";
string        name42 = "[Trade] EP21 Label";
string        name43 = "[Trade] EP22 Line";
string        name44 = "[Trade] EP22 Label";
string        name45 = "[Trade] EP23 Line";
string        name46 = "[Trade] EP23 Label";
string        name47 = "[Trade] EP24 Line";
string        name48 = "[Trade] EP24 Label";
string        name49 = "[Trade] EP25 Line";
string        name50 = "[Trade] EP25 Label";
string        name51 = "[Trade] EP26 Line";
string        name52 = "[Trade] EP26 Label";
string        name53 = "[Trade] EP27 Line";
string        name54 = "[Trade] EP27 Label";
string        name55 = "[Trade] EP28 Line";
string        name56 = "[Trade] EP28 Label";
string        name57 = "[Trade] EP29 Line";
string        name58 = "[Trade] EP29 Label";
string        name59 = "[Trade] EP30 Line";
string        name60 = "[Trade] EP30 Label";
string        name71 = "[Trade] Avg Line";
string        name72 = "[Trade] Avg Label";
string        name73 = "[Trade] TP1 Line";
string        name74 = "[Trade] TP1 Label";
string        name75 = "[Trade] SL1 Line";
string        name76 = "[Trade] SL1 Label";
string        name77 = "y[Trade] P/L Box";
string        name78 = "y[Trade] P/L Label";
string        name80;
string        name81;
string        name82;
string        name83;
string        name84;

//Profit & Loss
int           p,l;
string        Profit,Loss;
static double TradeClosedPrice;

//Labels
color         CurrentPL_Color;
string        Label_Style                     = "Arial Bold";  
int           Label_Size                      = 9;

//Arrows
double        Color, OrderPrice;
datetime      OrderTime;

//Chart_Zoom Global Variable Number
double        Chart_Zoom; 
string        Zoom;

//Automatic Zoom Parameters
string        note1                           = "Auto-Width_Adjustments:";
string        note2                           = "Wide monitors may require";
string        note3                           = "WindowsBarsPerChart be increased";
string        note4                           = "for these chart zoom settings:";
int           Zoom_Out_2_Bars                 = 1200;
int           Zoom_Out_1_Bars                 = 500;
int           Zoom_Std_Bars                   = 200;
int           Zoom_In_1_Bars                  = 100;
int           Zoom_In_2_Bars                  = 60;

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+ 
int init()
  {
  //Import Chart_Zoom Number
  if(Chart_Zoom_Number_01234567 == 0)
    {
    Zoom  = ("Zoom_Set"+Symbol()+Period());    
    Chart_Zoom = GlobalVariableGet(Zoom);
    }
  //Use Manually Set Chart_Zoom Number 
  else if(Chart_Zoom_Number_01234567 > 0 && Chart_Zoom_Number_01234567 < 7) 
      {
      Chart_Zoom = Chart_Zoom_Number_01234567;
      Zoom  = ("Zoom_Set"+Symbol()+Period());     
      GlobalVariableSet(Zoom, Chart_Zoom);      
      }
  //Auto-Sense Chart_Zoom Number    
  else if(Chart_Zoom_Number_01234567 == 7)
    {  
          if (WindowBarsPerChart( ) >= Zoom_Out_2_Bars) {Chart_Zoom=1;} 
    else {if (WindowBarsPerChart( ) >= Zoom_Out_1_Bars) {Chart_Zoom=2;} 
    else {if (WindowBarsPerChart( ) >= Zoom_Std_Bars)   {Chart_Zoom=3;} 
    else {if (WindowBarsPerChart( ) >= Zoom_In_1_Bars)  {Chart_Zoom=4;} 
    else {if (WindowBarsPerChart( ) >= Zoom_In_2_Bars)  {Chart_Zoom=5;} 
    else {Chart_Zoom=6;} }}}}
    }                      
  else
    {
    Alert("PVA Candles: Chart_Zoom out of range");
    return(0);
    }
  
  //Panel - determine display of Market Price  
  //if(Digits == 2) {Use_For_Forex = false;}          
  if(Digits == 5 || Digits == 3) {Factor = 10;} 
  else {Factor = 1;} //cater for 5 digits 
    
  //Panel and Levels - Check for unconventional Point digits number. 
  //Used for display of market price and Levels.
  if (Point == 0.00001) {Poin = 0.0001;} //5 digits     
  else {if (Point == 0.001) {Poin = 0.01;} //3 digits
  else {Poin = Point;}} //Normal 4 digits    
  if (StringSubstr(Symbol(),0,6) == "XAUUSD") {Poin = 1;}   
  else {if (StringSubstr(Symbol(),0,6) == "USDMXN") {Poin = 0.001;}
  else {if (StringSubstr(Symbol(),0,6) == "USDCZK") {Poin = 0.001;} }} 
  
  Deinitialized = false;
  ClosedFlag = false;  
  ShowTradeHistoryOnChart(true); 

  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+  
int deinit()
  {
  obj_total= ObjectsTotal();  
  for(k= obj_total; k>=0; k--)
    {
    name= ObjectName(k);
    if (StringSubstr(name,0,7)=="[Trade]"){ObjectDelete(name);}
    if (StringSubstr(name,0,8)=="y[Trade]"){ObjectDelete(name);}
    }        
  //Comment("");    
  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+ 
int start()
  {
  //If indicator is "Off" or chart TF is out of range deinitialize only once, not every tick.  
  if((!Indicator_On) || ((Period() < Display_Min_TF) || (Period() > Display_Max_TF)))    
    {
    if (!Deinitialized) {deinit(); Deinitialized = true;}
    return(0);
    }

  //Substitue Trade Levels from actual MT4 orders if available
  if(AutoFill_vs_Manual_Inputs == true)
    {
    updateTradeLevels(); //calls "UpdateAverage()" subroutine 
    if(__Show_Entry_and_Exit_Arrows) {DrawArrows();}
    if(__Show_EP_to_Close_Lines) {ShowTradeHistoryOnChart(false);}   
    }
  else
    {
    UpdateAverage();
    }
  
  //Calculate profit and loss based on EP and TP/SL values
  CalculateProfit();    
          
  //Set stop/start values for trade lines------------------------------------------------------
  T0=Time[0]; //all lines (anchor point)  
 
  //T1= individual EP lines stop/labels start   
  //T2= average EP line stop/label start (shifted right)
  //T3= TP and SL lines stop/labels start (P/L displayed)
  //T4= TP and SL lines stop/labels start (shifted right + P/L displayed)
  
  //T5= selector for avgEP line stop/label start (use T2 only) 
  //T6= selector for TP and SL lines stop  (use T1 or T2)    
  //T7= selector for TP and SL label start (use T3 or T4) 

  //Stop/start values for trade lines for each chart zoom                   
  if(Chart_Zoom == 1) 
    {
    T1=Time[0]+(Period()*60*20);
    T2=Time[0]+(Period()*60*114);               
    T3=Time[0]+(Period()*60*52); 
    T4=Time[0]+(Period()*60*148);  
    }     
  else if(Chart_Zoom == 2) 
    {
    T1=Time[0]+(Period()*60*10);   
    T2=Time[0]+(Period()*60*57);
    T3=Time[0]+(Period()*60*27); 
    T4=Time[0]+(Period()*60*74);            
    }   
  else if(Chart_Zoom == 3) 
    {
    T1=Time[0]+(Period()*60*4);              
    T2=Time[0]+(Period()*60*28);
    T3=Time[0]+(Period()*60*12);   
    T4=Time[0]+(Period()*60*36);     
    }
  else if(Chart_Zoom == 4) 
    {
    T1=Time[0]+(Period()*60*3);       
    T2=Time[0]+(Period()*60*15);
    T3=Time[0]+(Period()*60*7);
    T4=Time[0]+(Period()*60*19);    
    }
  else if(Chart_Zoom == 5) 
    {
    T1=Time[0]+(Period()*60*2);          
    T2=Time[0]+(Period()*60*8);
    T3=Time[0]+(Period()*60*4); 
    T4=Time[0]+(Period()*60*10);  
    }
  else if(Chart_Zoom == 6) 
    {
    T1=Time[0]+(Period()*60*2);       
    T2=Time[0]+(Period()*60*5); 
    T3=Time[0]+(Period()*60*3); 
    T4=Time[0]+(Period()*60*6);   
    }
 
  //Right Shift Management
  if(EP_2>0)
    {
    //right shift with P/L display with two or more EPs
    T5=T2;T6=T2;T7=T4;
    }  
  else
    {
    //no right shift but showing P/L display
    T6=T1;T7=T3;
    }      

  //Do Invidual EPs----------------------------------------------------------------------------
  if(EP_1 != 0 && EP_2 == 0)
    {
    ObjectDelete(name01);
    ObjectCreate(name01,OBJ_TREND,0,T1,EP_1,T0,EP_1);
    ObjectSet(name01,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name02);
    ObjectCreate(name02,OBJ_TEXT,0,T7,EP_1);
    EPstr = DoubleToStr(EP_1,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}    
    if(__Trade_is_Long_vs_Short) {EPstr = EPstr + "     long ";}
    else {EPstr = EPstr + "    short";}           
    ObjectSetText(name02,spc2+"EP 01:  " +EPstr,Label_Size,Label_Style,EP_Color);           
    }    
  else if(EP_1 != 0)
    {
    ObjectDelete(name01);
    ObjectCreate(name01,OBJ_TREND,0,T1,EP_1,T0,EP_1);
    ObjectSet(name01,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name02);
    ObjectCreate(name02,OBJ_TEXT,0,T1,EP_1); 
    EPstr = DoubleToStr(EP_1,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name02, spc+"EP 01:  " +EPstr, Label_Size, Label_Style, EP_Color);         
    }
  if(EP_2 != 0)
    {
    ObjectDelete(name03);
    ObjectCreate(name03,OBJ_TREND,0,T1,EP_2,T0,EP_2);
    ObjectSet(name03,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name04);
    ObjectCreate(name04,OBJ_TEXT,0,T1,EP_2);     
    EPstr = DoubleToStr(EP_2,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name04, spc+"EP 02:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }      
  if(EP_3 != 0)
    {
    ObjectDelete(name05);
    ObjectCreate(name05,OBJ_TREND,0,T1,EP_3,T0,EP_3);
    ObjectSet(name05,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name06);
    ObjectCreate(name06,OBJ_TEXT,0,T1,EP_3);     
    EPstr = DoubleToStr(EP_3,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name06, spc+"EP 03:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }
  if(EP_4 != 0)
    {
    ObjectDelete(name07);
    ObjectCreate(name07,OBJ_TREND,0,T1,EP_4,T0,EP_4);
    ObjectSet(name07,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name08);
    ObjectCreate(name08,OBJ_TEXT,0,T1,EP_4);     
    EPstr = DoubleToStr(EP_4,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name08, spc+"EP 04:  " +EPstr, Label_Size, Label_Style, EP_Color);      
    }      
  if(EP_5 != 0)
    {
    ObjectDelete(name09);
    ObjectCreate(name09,OBJ_TREND,0,T1,EP_5,T0,EP_5);
    ObjectSet(name09,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name10);
    ObjectCreate(name10,OBJ_TEXT,0,T1,EP_5);     
    EPstr = DoubleToStr(EP_5,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name10, spc+"EP 05:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }         
  if(EP_6 != 0)
    {
    ObjectDelete(name11);
    ObjectCreate(name11,OBJ_TREND,0,T1,EP_6,T0,EP_6);
    ObjectSet(name11,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name12);
    ObjectCreate(name12,OBJ_TEXT,0,T1,EP_6);     
    EPstr = DoubleToStr(EP_6,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name12, spc+"EP 06:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }
  if(EP_7 != 0)
    {
    ObjectDelete(name13);
    ObjectCreate(name13,OBJ_TREND,0,T1,EP_7,T0,EP_7);
    ObjectSet(name13,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name14);
    ObjectCreate(name14,OBJ_TEXT,0,T1,EP_7);     
    EPstr = DoubleToStr(EP_7,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name14, spc+"EP 07:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }      
  if(EP_8 != 0)
    {
    ObjectDelete(name15);
    ObjectCreate(name15,OBJ_TREND,0,T1,EP_8,T0,EP_8);
    ObjectSet(name15,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name16);
    ObjectCreate(name16,OBJ_TEXT,0,T1,EP_8);     
    EPstr = DoubleToStr(EP_8,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name16, spc+"EP 08:  " +EPstr, Label_Size, Label_Style, EP_Color);  
    }         
  if(EP_9 != 0)
    {
    ObjectDelete(name17);
    ObjectCreate(name17,OBJ_TREND,0,T1,EP_9,T0,EP_9);
    ObjectSet(name17,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name18);
    ObjectCreate(name18,OBJ_TEXT,0,T1,EP_9);     
    EPstr = DoubleToStr(EP_9,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name18, spc+"EP 09:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }        
  if(EP_10 != 0)
    {
    ObjectDelete(name19);
    ObjectCreate(name19,OBJ_TREND,0,T1,EP_10,T0,EP_10);
    ObjectSet(name19,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name20);
    ObjectCreate(name20,OBJ_TEXT,0,T1,EP_10);     
    EPstr = DoubleToStr(EP_10,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name20, spc+"EP 10:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }
  if(EP_11 != 0)
    {
    ObjectDelete(name21);
    ObjectCreate(name21,OBJ_TREND,0,T1,EP_11,T0,EP_11);
    ObjectSet(name21,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name22);
    ObjectCreate(name22,OBJ_TEXT,0,T1,EP_11);     
    EPstr = DoubleToStr(EP_11,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name22, spc+"EP 11:  " +EPstr, Label_Size, Label_Style, EP_Color);         
    }      
  if(EP_12 != 0)
    {
    ObjectDelete(name23);
    ObjectCreate(name23,OBJ_TREND,0,T1,EP_12,T0,EP_12);
    ObjectSet(name23,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name24);
    ObjectCreate(name24,OBJ_TEXT,0,T1,EP_12);     
    EPstr = DoubleToStr(EP_12,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name24, spc+"EP 12:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }
  if(EP_13 != 0)
    {
    ObjectDelete(name25);
    ObjectCreate(name25,OBJ_TREND,0,T1,EP_13,T0,EP_13);
    ObjectSet(name25,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name26);
    ObjectCreate(name26,OBJ_TEXT,0,T1,EP_13);     
    EPstr = DoubleToStr(EP_13,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name26, spc+"EP 13:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }      
  if(EP_14 != 0)
    {
    ObjectDelete(name27);
    ObjectCreate(name27,OBJ_TREND,0,T1,EP_14,T0,EP_14);
    ObjectSet(name27,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name28);
    ObjectCreate(name28,OBJ_TEXT,0,T1,EP_14);     
    EPstr = DoubleToStr(EP_14,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name28, spc+"EP 14:  " +EPstr, Label_Size, Label_Style, EP_Color);    
    }         
  if(EP_15 != 0)
    {
    ObjectDelete(name29);
    ObjectCreate(name29,OBJ_TREND,0,T1,EP_15,T0,EP_15);
    ObjectSet(name29,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name30);
    ObjectCreate(name30,OBJ_TEXT,0,T1,EP_15);     
    EPstr = DoubleToStr(EP_15,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name30, spc+"EP 15:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }                                                                     
  if(EP_16 != 0)
    {
    ObjectDelete(name31);
    ObjectCreate(name31,OBJ_TREND,0,T1,EP_16,T0,EP_16);
    ObjectSet(name31,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name32);
    ObjectCreate(name32,OBJ_TEXT,0,T1,EP_16);     
    EPstr = DoubleToStr(EP_16,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name32, spc+"EP 16:  " +EPstr, Label_Size, Label_Style, EP_Color);     
    }                                                                     
  if(EP_17 != 0)
    {
    ObjectDelete(name33);
    ObjectCreate(name33,OBJ_TREND,0,T1,EP_17,T0,EP_17);
    ObjectSet(name33,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name34);
    ObjectCreate(name34,OBJ_TEXT,0,T1,EP_17);     
    EPstr = DoubleToStr(EP_17,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name34, spc+"EP 17:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }                      
  if(EP_18 != 0)
    {
    ObjectDelete(name35);
    ObjectCreate(name35,OBJ_TREND,0,T1,EP_18,T0,EP_18);
    ObjectSet(name35,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name36);
    ObjectCreate(name36,OBJ_TEXT,0,T1,EP_18);     
    EPstr = DoubleToStr(EP_18,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name36, spc+"EP 18:  " +EPstr, Label_Size, Label_Style, EP_Color);   
    }
  if(EP_19 != 0)
    {
    ObjectDelete(name37);
    ObjectCreate(name37,OBJ_TREND,0,T1,EP_19,T0,EP_19);
    ObjectSet(name37,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name38);
    ObjectCreate(name38,OBJ_TEXT,0,T1,EP_19);     
    EPstr = DoubleToStr(EP_19,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name38, spc+"EP 19:  " +EPstr, Label_Size, Label_Style, EP_Color);  
    }
  if(EP_20 != 0)
    {
    ObjectDelete(name39);
    ObjectCreate(name39,OBJ_TREND,0,T1,EP_20,T0,EP_20);
    ObjectSet(name39,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name40);
    ObjectCreate(name40,OBJ_TEXT,0,T1,EP_20);     
    EPstr = DoubleToStr(EP_20,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name40, spc+"EP 20:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }                                                                                                                                                                                                                                    
  if(EP_21 != 0)
    {
    ObjectDelete(name41);
    ObjectCreate(name41,OBJ_TREND,0,T1,EP_21,T0,EP_21);
    ObjectSet(name41,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name42);
    ObjectCreate(name42,OBJ_TEXT,0,T1,EP_21);     
    EPstr = DoubleToStr(EP_21,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name42, spc+"EP 21:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }      
  if(EP_22 != 0)
    {
    ObjectDelete(name43);
    ObjectCreate(name43,OBJ_TREND,0,T1,EP_22,T0,EP_22);
    ObjectSet(name43,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name44);
    ObjectCreate(name44,OBJ_TEXT,0,T1,EP_22);     
    EPstr = DoubleToStr(EP_22,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name44, spc+"EP 22:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }   
  if(EP_23 != 0)
    {
    ObjectDelete(name45);
    ObjectCreate(name45,OBJ_TREND,0,T1,EP_23,T0,EP_23);
    ObjectSet(name45,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name46);
    ObjectCreate(name46,OBJ_TEXT,0,T1,EP_23);     
    EPstr = DoubleToStr(EP_23,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name46, spc+"EP 23:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }        
  if(EP_24 != 0)
    {
    ObjectDelete(name47);
    ObjectCreate(name47,OBJ_TREND,0,T1,EP_24,T0,EP_24);
    ObjectSet(name47,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name48);
    ObjectCreate(name48,OBJ_TEXT,0,T1,EP_24);     
    EPstr = DoubleToStr(EP_24,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name48, spc+"EP 24:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }        
  if(EP_25 != 0)
    {
    ObjectDelete(name49);
    ObjectCreate(name49,OBJ_TREND,0,T1,EP_25,T0,EP_25);
    ObjectSet(name49,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name50);
    ObjectCreate(name50,OBJ_TEXT,0,T1,EP_25);     
    EPstr = DoubleToStr(EP_25,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name50, spc+"EP 25:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }                
  if(EP_26 != 0)
    {
    ObjectDelete(name51);
    ObjectCreate(name51,OBJ_TREND,0,T1,EP_26,T0,EP_26);
    ObjectSet(name51,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name52);
    ObjectCreate(name52,OBJ_TEXT,0,T1,EP_26);     
    EPstr = DoubleToStr(EP_26,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name52, spc+"EP 26:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }      
  if(EP_27 != 0)
    {
    ObjectDelete(name53);
    ObjectCreate(name53,OBJ_TREND,0,T1,EP_27,T0,EP_27);
    ObjectSet(name53,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name54);
    ObjectCreate(name54,OBJ_TEXT,0,T1,EP_27);     
    EPstr = DoubleToStr(EP_27,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name54, spc+"EP 27:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }   
  if(EP_28 != 0)
    {
    ObjectDelete(name55);
    ObjectCreate(name55,OBJ_TREND,0,T1,EP_28,T0,EP_28);
    ObjectSet(name55,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name56);
    ObjectCreate(name56,OBJ_TEXT,0,T1,EP_28);     
    EPstr = DoubleToStr(EP_28,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name56, spc+"EP 28:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }        
  if(EP_29 != 0)
    {
    ObjectDelete(name57);
    ObjectCreate(name57,OBJ_TREND,0,T1,EP_29,T0,EP_29);
    ObjectSet(name57,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name58);
    ObjectCreate(name58,OBJ_TEXT,0,T1,EP_29);     
    EPstr = DoubleToStr(EP_29,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name58, spc+"EP 29:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }        
  if(EP_30 != 0)
    {
    ObjectDelete(name59);
    ObjectCreate(name59,OBJ_TREND,0,T1,EP_30,T0,EP_30);
    ObjectSet(name59,OBJPROP_COLOR,EP_Color);
    ObjectDelete(name60);
    ObjectCreate(name60,OBJ_TEXT,0,T1,EP_30);     
    EPstr = DoubleToStr(EP_30,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
    ObjectSetText(name60, spc+"EP 30:  " +EPstr, Label_Size, Label_Style, EP_Color); 
    }                
                                        
  //Do Average of EPs-------------------------------------------------------------------------- 
  if(EP_2>0)
    {
    ObjectDelete(name71);
    ObjectCreate(name71,OBJ_TREND,0,T5,Avg_Price,T0,Avg_Price);
    ObjectSet(name71,OBJPROP_COLOR,EP_Av_Color);
    ObjectDelete(name72);
    ObjectCreate(name72,OBJ_TEXT,0,T7,Avg_Price);
    EPstr = DoubleToStr(Avg_Price,Digits);
    if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}    
    if(__Trade_is_Long_vs_Short) {EPstr = EPstr + "     long ";}
    else {EPstr = EPstr + "    short";}  
    if(Trades <10)
      {          
      ObjectSetText(name72,spc2+"Av 0"+Trades+":  " +EPstr,Label_Size,Label_Style,EP_Av_Color);    
      }      
    else
      {
      ObjectSetText(name72,spc2+"Av "+Trades+":  " +EPstr,Label_Size,Label_Style,EP_Av_Color);
      }            
    }

  //Do TP-------------------------------------------------------------------------------------- 
  ObjectDelete(name73);    
  ObjectCreate(name73,OBJ_TREND,0,T6,TP_1,T0,TP_1);
  ObjectSet(name73,OBJPROP_COLOR,TP_Color);
  EPstr = DoubleToStr(TP_1,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectDelete(name74);
  ObjectCreate(name74,OBJ_TEXT,0,T7,TP_1);
  ObjectSetText(name74, spc+"TP 01:  " +EPstr + Profit, Label_Size, Label_Style, TP_Color);  
              
  //Do SL--------------------------------------------------------------------------------------
  if(SL_1 != 0)  
  ObjectDelete(name75);   
  ObjectCreate(name75,OBJ_TREND,0,T6,SL_1,T0,SL_1);
  ObjectSet(name75,OBJPROP_COLOR,SL_Color);    
  EPstr = DoubleToStr(SL_1,Digits);
  if(StringLen(EPstr)==6) {EPstr= "0"+ EPstr;}
  ObjectDelete(name76);
  ObjectCreate(name76,OBJ_TEXT,0,T7,SL_1);
  ObjectSetText(name76, spc+"SL 01:  " +EPstr + Loss, Label_Size, Label_Style, SL_Color); 
             
  //Do CurrentPL-------------------------------------------------------------------------------
  ObjectDelete(name77);
  ObjectDelete(name78);    
  if( (EP_1 != 0) && ((AutoFill_vs_Manual_Inputs && ClosedFlag == false) || 
    (AutoFill_vs_Manual_Inputs == false && (CurrentPL<PositionProfit && CurrentPL>PositionLoss))))         
    {
    if(CurrentPL <0) {EPstr = "P/L:  " + DoubleToStr(CurrentPL,1) + "  pips";}
    else {EPstr = " P/L: +" + DoubleToStr(CurrentPL,1) + "  pips";}
    spc3= ""; for(i=2; i<=StringLen(EPstr); i++) {spc3= spc3 + "g";}
    ObjectCreate(name77,OBJ_TEXT,0,Time[WindowFirstVisibleBar()],Bid);
    ObjectSetText(name77,spc3, 11, "Webdings",Current_PL_Background_Color);   
    if(CurrentPL >= 0) {CurrentPL_Color = TP_Color;}
    else {CurrentPL_Color = SL_Color;}   
    spc3= ""; for(i=3; i<=StringLen(EPstr); i++) {spc3= spc3 + "  ";}
    ObjectCreate(name78,OBJ_TEXT,0,Time[WindowFirstVisibleBar()],Bid);
    ObjectSetText(name78,spc3+EPstr,10,"tahoma bold",CurrentPL_Color);
    }           

  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine: Zero all trade levels and refill based on actual MT4 orders executed          |                                                        
//+-------------------------------------------------------------------------------------------+ 
int updateTradeLevels()
  {
  //Count total number of open orders in current symbol and fill orderList arrays.  The content
  //of the arrays is maintained after the close of all orders and will continue to be displayed
  //so that a picture of the closed trade can be obtained if the indicator remains set to
  //Autofill, if no input is changed, and if the chart TF remains unchanged.  Once any input is
  //changed or the chart TF is changed, MT4 will deinitialize, clearing all data.   

  //Fill arrays from any open orders found and update "orders" count
  orders=0;  
  for(i=OrdersTotal(); i>=0; i--) 
    {
    if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
      {
      if (Symbol() == OrderSymbol() && (OrderType() == OP_BUY || OrderType() == OP_SELL)) 
        {
        orderList[orders][0] = OrderOpenTime();
        orderList[orders][1] = OrderOpenPrice();
        orders++;
        if(OrderType() == OP_BUY) {__Trade_is_Long_vs_Short = true;}
        else if(OrderType() == OP_SELL) {__Trade_is_Long_vs_Short = false;} 
        if(!__Apply_Manual_SL_TP)
          {       
          TP = OrderTakeProfit( ); 
          SL = OrderStopLoss( ); 
          }           
        }
      } 
    }

  //If there are no open orders for the selected symbol, show trades from history.
  if(orders == 0)
    {
    for(i=0;i<OrdersHistoryTotal();i++)
      {
      if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
        {
        Print("History access failed!");
        break;
        }
     
      //Only show trades which are closed within the last __Show_Trade_History_Limit minutes.
      //Because datetime is calculated in seconds, the variable has to be multiplied by 60
      if(Symbol() == OrderSymbol() && (OrderType() == OP_BUY || OrderType() == OP_SELL) 
        && OrderCloseTime() >= (TimeCurrent()-(____Show_Lines_for_Minutes * 60)))
        {
        orderList[orders][0] = OrderOpenTime();
        orderList[orders][1] = OrderOpenPrice();
        orders++;
        if(OrderType() == OP_BUY) {__Trade_is_Long_vs_Short = true;}
        else if(OrderType() == OP_SELL) {__Trade_is_Long_vs_Short = false;} 
        if(!__Apply_Manual_SL_TP)
          {       
          TP = OrderTakeProfit( ); 
          SL = OrderStopLoss( ); 
          }
        }
      }
    }
  
  //If there are open orders, clear all EPs and replace with array datum.  When the trade is
  //closed the arrays remain filled and the chart will continue to display the AutoFill trade 
  //levels so long as nothing is changed to cause MT4 to deinitialize, clearing the array data. 
  if(orders > 0)
    {       
    //Clear EPs
    EP_1  = 0;
    EP_2  = 0;
    EP_3  = 0;
    EP_4  = 0;
    EP_5  = 0;
    EP_6  = 0;
    EP_7  = 0;
    EP_8  = 0;
    EP_9  = 0;
    EP_10 = 0;
    EP_11 = 0;
    EP_12 = 0;
    EP_13 = 0;
    EP_14 = 0;
    EP_15 = 0;
    EP_16 = 0;
    EP_17 = 0;
    EP_18 = 0;
    EP_19 = 0;
    EP_20 = 0;
    EP_21 = 0;
    EP_22 = 0;
    EP_23 = 0;
    EP_24 = 0;
    EP_25 = 0;  
    EP_26 = 0;
    EP_27 = 0;
    EP_28 = 0;
    EP_29 = 0;
    EP_30 = 0;       
    ArrayResize(orderList,orders);
    ArraySort(orderList);    
    //Re-set EPs to OrderOpenPrice()
    for(i=1; i<=orders; i++) 
      {
      switch(i)
        {
        case  1:  EP_1  = orderList[i-1][1]; break;
        case  2:  EP_2  = orderList[i-1][1]; break;
        case  3:  EP_3  = orderList[i-1][1]; break;
        case  4:  EP_4  = orderList[i-1][1]; break;
        case  5:  EP_5  = orderList[i-1][1]; break;
        case  6:  EP_6  = orderList[i-1][1]; break;
        case  7:  EP_7  = orderList[i-1][1]; break;
        case  8:  EP_8  = orderList[i-1][1]; break;
        case  9:  EP_9  = orderList[i-1][1]; break;
        case 10:  EP_10 = orderList[i-1][1]; break;
        case 11:  EP_11 = orderList[i-1][1]; break;
        case 12:  EP_12 = orderList[i-1][1]; break;
        case 13:  EP_13 = orderList[i-1][1]; break;
        case 14:  EP_14 = orderList[i-1][1]; break;
        case 15:  EP_15 = orderList[i-1][1]; break;
        case 16:  EP_16 = orderList[i-1][1]; break;
        case 17:  EP_17 = orderList[i-1][1]; break;
        case 18:  EP_18 = orderList[i-1][1]; break;
        case 19:  EP_19 = orderList[i-1][1]; break;
        case 20:  EP_20 = orderList[i-1][1]; break;
        case 21:  EP_21 = orderList[i-1][1]; break;
        case 22:  EP_22 = orderList[i-1][1]; break;
        case 23:  EP_23 = orderList[i-1][1]; break;
        case 24:  EP_24 = orderList[i-1][1]; break;
        case 25:  EP_25 = orderList[i-1][1]; break;        
        case 26:  EP_26 = orderList[i-1][1]; break;
        case 27:  EP_27 = orderList[i-1][1]; break;
        case 28:  EP_28 = orderList[i-1][1]; break;
        case 29:  EP_29 = orderList[i-1][1]; break;
        case 30:  EP_30 = orderList[i-1][1]; break;         
        } 
      }
      
    //Calculate average based on either manually or autofilled inputs
    UpdateAverage();       
      
    if(!__Apply_Manual_SL_TP)
      {
      if(TradeClosedPrice == 0)
        {       
        TP_1 = TP;
        SL_1 = SL;
        }
      else //if(TradeClosedPrice != 0)
        {
        if(__Trade_is_Long_vs_Short) 
          {
          if(TradeClosedPrice >= Avg_Price) 
            {
            TP_1 = TradeClosedPrice;
            SL_1 = EMPTY_VALUE;
            }
          else
            {
            SL_1 = TradeClosedPrice;
            TP_1 = EMPTY_VALUE;
            }
          } 
        else //if(!__Trade_is_Long_vs_Short && TradeClosedPrice <= Avg_Price) 
          {
          if(TradeClosedPrice <= Avg_Price) 
            {
            TP_1 = TradeClosedPrice;
            SL_1 = EMPTY_VALUE;
            }
          else 
            {
            SL_1 = TradeClosedPrice;
            TP_1 = EMPTY_VALUE;
            }
          } 
        }
      }             
    }
  
  return(0);
  }
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine: Calculate average price based on manually entered or auto filled EPs          |                                                        
//+-------------------------------------------------------------------------------------------+ 
void UpdateAverage()
  {  
  //Find average
  if(EP_30 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24 + EP_25 + EP_26 + EP_27 + EP_28 + EP_29 + EP_30)/30;
    Trades = 30;     
    }       
  else {if(EP_29 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24 + EP_25 + EP_26+ EP_27 + EP_28 + EP_29)/29;
    Trades = 29;     
    }       
  else {if(EP_28 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24 + EP_25 + EP_26 + EP_27 + EP_28)/28;
    Trades = 28;     
    }       
  else {if(EP_27 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24 + EP_25 + EP_26 + EP_27)/27;
    Trades = 27;     
    }      
  else {if(EP_26 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24 + EP_25 + EP_26)/26;
    Trades = 26;     
    }           
  else {if(EP_25 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24 + EP_25)/25;
    Trades = 25;     
    }             
  else {if(EP_24 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23 + EP_24)/24;
    Trades = 24;     
    }           
  else {if(EP_23 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22 + EP_23)/23;
    Trades = 23;     
    }         
  else {if(EP_22 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21 + EP_22)/22;
    Trades = 22;     
    }        
  else {if(EP_21 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20 
    + EP_21)/21;
    Trades = 21;     
    }      
  else {if(EP_20 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19 + EP_20)/20;
    Trades = 20;     
    }          
  else {if(EP_19 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18 + EP_19)/19;
    Trades = 19;     
    }       
  else {if(EP_18 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17 + EP_18)/18;
    Trades = 18;     
    }    
  else {if(EP_17 > 0) 
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16 + EP_17)/17;
    Trades = 17;     
    }
  else {if(EP_16 > 0)      
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15 + EP_16)/16;
    Trades = 16;   
    }    
  else {if(EP_15 > 0)      
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14 + EP_15)/15;
    Trades = 15;        
    }    
  else {if(EP_14 > 0)      
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13 + EP_14)/14;
    Trades = 14;        
    }      
  else {if(EP_13 > 0)      
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12 + EP_13)/13;
    Trades = 13;        
    }    
  else {if(EP_12 > 0)      
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11 + EP_12)/12;
    Trades = 12;                               
    }
  else {if(EP_11 > 0)       
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10
    + EP_11)/11;
    Trades = 11;                               
    }   
  else {if(EP_10 > 0)       
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9 + EP_10)/10;
    Trades = 10;                               
    }   
  else {if(EP_9 > 0)       
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8 + EP_9)/9;
    Trades = 9;                               
    } 
  else {if(EP_8 > 0)       
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7 + EP_8)/8;
    Trades = 8;
    }
  else {if(EP_7 > 0)       
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6 + EP_7)/7;
    Trades = 7;
    }
  else {if(EP_6 > 0)   
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5 + EP_6)/6;
    Trades = 6;
    }          
  else {if(EP_5 > 0)    
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4 + EP_5)/5;
    Trades = 5;
    }            
  else {if(EP_4 > 0)  
    {
    Avg_Price = (EP_1 + EP_2 + EP_3 + EP_4)/4;
    Trades = 4;
    }
  else {if(EP_3 > 0)     
    {
    Avg_Price = (EP_1 + EP_2 + EP_3)/3;
    Trades = 3;
    }
  else {if(EP_2 > 0)     
    {
    Avg_Price = (EP_1 + EP_2)/2;
    Trades = 2;
    }    
  else {if(EP_1 > 0) 
    {
    Avg_Price= EP_1;
    Trades = 1;
    }                                     
    }}}}}}}}}}}}}}}}}}}}}}}}}}}}}            
  return;          
  }   
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine: Draw arrows for entry points                                                  |                                                        
//+-------------------------------------------------------------------------------------------+            
void DrawArrows()
  {
  //Set parameters based on long or short trade
  if(__Trade_is_Long_vs_Short)
    {
    name81 = "[Trade] Long";
    Color= __Arrow_Trade_Long_Color;
    }
  else
    {
    name81 = "[Trade] Short";
    Color= __Arrow_Trade_Short_Color;
    }  

  //Set parameters for arrows at EPs
  for(i=1; i<=orders; i++) 
    {
    OrderTime = orderList[orders-i][0];
    OrderPrice = orderList[orders-i][1];
    name84 = name81 + (orders-i+1);
    ObjectDelete(name84); 
    ObjectCreate(name84, OBJ_ARROW, 0, OrderTime, OrderPrice);     
    ObjectSet(name84, OBJPROP_ARROWCODE, 2); 
    ObjectSet(name84, OBJPROP_COLOR, Color);
    }
  return;
  }

//+-------------------------------------------------------------------------------------------+
//| Subroutine: Draw entry to close price connecting lines                                    |                                                        
//+-------------------------------------------------------------------------------------------+  
void ShowTradeHistoryOnChart(bool update)
  { 
  // Check if the update flag is set and reset the historyTotal counter if it is
  // This is just a performance improvement, so that history is not drawn on every single tick
  if (update) historyTotal = 0;
  
  if(historyTotal == OrdersHistoryTotal())
    {
    return;
    }
   
  // If the number of closed orders changed, draw lines and update the counter
  historyTotal = OrdersHistoryTotal();
  Print("## Run the loop ##"); 
  for(i=0;i<historyTotal;i++)
    {
    if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
      {
      Print("History access failed!");
      break;
      }
     
    // Only show trades which are closed within the last __Show_Trade_History_Limit minutes.
    // Because datetime is calculated in seconds, the variable has to be multiplied by 60
    if(Symbol() == OrderSymbol() && (OrderType() == OP_BUY || OrderType() == OP_SELL) && 
      OrderCloseTime() >= (TimeCurrent()-(____Show_Lines_for_Minutes * 60)))
      {     
      // Arrow at EP Price
      name81 = "[Trade] Arrow at EP " + OrderTicket();    
      ObjectCreate(name81, OBJ_ARROW, 0, OrderOpenTime(), OrderOpenPrice());
      ObjectSet(name81, OBJPROP_ARROWCODE, 2); 
      
      // Arrow at closing Price
      name82 = "[Trade] Arrow at Close " + OrderTicket();    
      ObjectCreate(name82, OBJ_ARROW, 0, OrderCloseTime(), OrderClosePrice());
      ObjectSet(name82, OBJPROP_ARROWCODE, 3); 
      ObjectSet(name82, OBJPROP_COLOR, __Arrow_Trade_Closed_Color);
          
      // Trendline from EP to Close
      name83 = "[Trade] Line from EP to Close " + OrderTicket();        
      ObjectCreate(name83, OBJ_TREND, 0, OrderOpenTime(), OrderOpenPrice(), 
        OrderCloseTime(), OrderClosePrice());
        
      // Set the color of the trendline depending on the order type (sell vs buy)
      if(OrderType() == OP_BUY) 
        {
       ObjectSet(name83, OBJPROP_COLOR, __Arrow_Trade_Long_Color);
       ObjectSet(name81, OBJPROP_COLOR, __Arrow_Trade_Long_Color);
        }
      else
        {
        ObjectSet(name83, OBJPROP_COLOR, __Arrow_Trade_Short_Color);
        ObjectSet(name81, OBJPROP_COLOR, __Arrow_Trade_Short_Color);
        }
      
      ObjectSet(name83, OBJPROP_STYLE, 2);
      ObjectSet(name83, OBJPROP_WIDTH, 1);
      ObjectSet(name83, OBJPROP_RAY, false);
            
      if(OrderClosePrice() >0) 
        {
        ClosedFlag = true;
        TradeClosedPrice = OrderClosePrice();
        }
      }     
    } 
  }
  
//+-------------------------------------------------------------------------------------------+
//| Subroutine: Profit/Loss based on auto filled EPs and all trades closed at TP1 or SL1      |                                                        
//+-------------------------------------------------------------------------------------------+ 
void CalculateProfit()
  {
  TP_1 = NormalizeDouble(TP_1,Digits);  
  SL_1 = NormalizeDouble(SL_1,Digits);  
  Avg_Price = NormalizeDouble(Avg_Price,Digits); 
  
  if(__Trade_is_Long_vs_Short)
    {
    PositionProfit= NormalizeDouble((((TP_1 - Avg_Price)*Trades)/Point)/Factor,Digits);
    PositionLoss = NormalizeDouble((((SL_1 - Avg_Price)*Trades)/Point)/Factor,Digits); 
    CurrentPL = (Bid/Poin - Avg_Price/Poin)*Trades;        
    }
  else
    {
    PositionProfit= NormalizeDouble((((Avg_Price - TP_1)*Trades)/Point)/Factor,Digits);
    PositionLoss = NormalizeDouble((((Avg_Price - SL_1)*Trades)/Point)/Factor,Digits); 
    CurrentPL = (Avg_Price/Poin - Ask/Poin)*Trades;   
    }
    
  if(TP_1 == 0) PositionProfit = +99999;
  if(SL_1 == 0) PositionLoss = -99999;
  Profit = DoubleToStr(PositionProfit,0);
  Loss = DoubleToStr(PositionLoss,0); 
  
  p = StringLen(Profit);
  l = StringLen(Loss); 
    
  if(PositionProfit >= 0) 
    {
    if(p == 1){Profit = "   ("+"000"+StringSubstr(Profit,0)+" p)     ";}  
    if(p == 2){Profit = "   ("+"00"+StringSubstr(Profit,0)+" p)     ";}  
    if(p == 3){Profit = "   ("+"0"+StringSubstr(Profit,0)+" p)     ";}  
    if(p == 4){Profit = "   ("+StringSubstr(Profit,0)+" p)     ";}    
    if(p == 5){Profit = "   ("+StringSubstr(Profit,0)+" p)  ";}     
    }   
  else 
    {
    if(p == 2){Profit = "   ("+ StringSubstr(Profit,0,1)+"000"+StringSubstr(Profit,1)+" p)     ";}  
    if(p == 3){Profit = "   ("+ StringSubstr(Profit,0,1)+"00"+StringSubstr(Profit,1)+" p)     ";}   
    if(p == 4){Profit = "   ("+ StringSubstr(Profit,0,1)+"0"+StringSubstr(Profit,1)+" p)     ";}  
    if(p == 5){Profit = "   ("+ StringSubstr(Profit,0)+" p)     ";}        
    }
  if(PositionLoss >= 0) 
    {
    if(l == 1){Loss = "   ("+"000"+StringSubstr(Loss,0)+" p)     ";}  
    if(l == 2){Loss = "   ("+"00"+StringSubstr(Loss,0)+" p)     ";}   
    if(l == 3){Loss = "   ("+"0"+StringSubstr(Loss,0)+" p)     ";}  
    if(l == 4){Loss = "   ("+StringSubstr(Loss,0)+" p)     ";}
    if(l == 5){Loss = "   ("+StringSubstr(Loss,0)+" p)  ";} 
    }     
  else 
    {  
    if(l == 2){Loss = "   ("+ StringSubstr(Loss,0,1)+"000"+StringSubstr(Loss,1)+" p)     ";} 
    if(l == 3){Loss = "   ("+ StringSubstr(Loss,0,1)+"00"+StringSubstr(Loss,1)+" p)     ";} 
    if(l == 4){Loss = "   ("+ StringSubstr(Loss,0,1)+"0"+StringSubstr(Loss,1)+" p)     ";} 
    if(l == 5){Loss = "   ("+ StringSubstr(Loss,0)+" p)     ";} 
    }    
  return;          
  }       
  
//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+